(*************************************************************************)
(*                                                                       *)
(*                                OCaPIC                                 *)
(*                                                                       *)
(*            Benoit Vaugon, Université Pierre et Marie Curie            *)
(*                                                                       *)
(*    Ce fichier est distribué sous les termes de la licence CeCILL-B    *)
(*    décrite dans le fichier ../../LICENCE.                             *)
(*                                                                       *)
(*************************************************************************)

type port = PORTA | PORTB | PORTC | PORTD | PORTE
val string_of_port : port -> string
val port_of_string : string -> port
val char_of_port : port -> char
val port_of_char : char -> port

type pin =
  | RA0 | RA1 | RA2 | RA3 | RA4 | RA5 | RA6 | RA7
  | RB0 | RB1 | RB2 | RB3 | RB4 | RB5 | RB6 | RB7
  | RC0 | RC1 | RC2 | RC3 | RC4 | RC5 | RC6 | RC7
  | RD0 | RD1 | RD2 | RD3 | RD4 | RD5 | RD6 | RD7
  | RE0 | RE1 | RE2 | RE3 | RE4 | RE5 | RE6 | RE7
val string_of_pin : pin -> string
val port_of_pin : pin -> port
val index_of_pin : pin -> int
val pin_of_port_index : port -> int -> pin
val pin_of_string : string -> pin

type handler =
  | Exit_handler of (unit -> unit)
  | Write_handler of (port -> int -> unit)
  | Write_port_handler of port * (int -> unit)
  | Tris_handler of (port -> int -> unit)
  | Tris_port_handler of port * (int -> unit)
  | Set_handler of (pin -> unit)
  | Clear_handler of (pin -> unit)
  | Change_handler of (pin -> bool -> unit)
  | Set_pin_handler of pin * (unit -> unit)
  | Clear_pin_handler of pin * (unit -> unit)
  | Change_pin_handler of pin * (bool -> unit)
  | Setin_handler of (pin -> unit)
  | Setout_handler of (pin -> unit)
  | Setstate_handler of (pin -> bool -> unit)
  | Setin_pin_handler of pin * (unit -> unit)
  | Setout_pin_handler of pin * (unit -> unit)
  | Setstate_pin_handler of pin * (bool -> unit)

val add_handler : handler -> unit
val remove_handler : handler -> unit

val start : unit -> unit
val join : unit -> unit

val write_port : port -> int -> unit
val set_pin : pin -> unit
val clear_pin : pin -> unit
val change_pin : pin -> bool -> unit

val read_port : port -> int
val read_tris : port -> int
val test_pin : pin -> bool
